/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.bluez.GattService1.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef GATTSERVICE1ADAPTOR_P_H
#define GATTSERVICE1ADAPTOR_P_H

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API.  It exists purely as an
// implementation detail.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
//

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "bluez5_helper_p.h"
#include <QtCore/qcontainerfwd.h>

namespace QtBluetoothPrivate {

/*
 * Adaptor class for interface org.bluez.GattService1
 */
class OrgBluezGattService1Adaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.bluez.GattService1")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.bluez.GattService1\">\n"
"    <property access=\"read\" type=\"s\" name=\"UUID\"/>\n"
"    <property access=\"read\" type=\"o\" name=\"Device\"/>\n"
"    <property access=\"read\" type=\"b\" name=\"Primary\"/>\n"
"    <property access=\"read\" type=\"ao\" name=\"Includes\"/>\n"
"  </interface>\n"
        "")
public:
    OrgBluezGattService1Adaptor(QObject *parent);
    ~OrgBluezGattService1Adaptor() override;

public: // PROPERTIES
    Q_PROPERTY(QDBusObjectPath Device READ device)
    QDBusObjectPath device() const;

    Q_PROPERTY(QList<QDBusObjectPath> Includes READ includes)
    QList<QDBusObjectPath> includes() const;

    Q_PROPERTY(bool Primary READ primary)
    bool primary() const;

    Q_PROPERTY(QString UUID READ uUID)
    QString uUID() const;

public Q_SLOTS: // METHODS
Q_SIGNALS: // SIGNALS
};

} // end of namespace QtBluetoothPrivate

#endif
