// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#ifndef SERVICEMAP_P_H
#define SERVICEMAP_P_H

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API.  It exists purely as an
// implementation detail.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
//

#include <QtBluetooth/qtbluetoothglobal.h>

#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtDBus/QDBusArgument>
#include <QtCore/private/qglobal_p.h>

QT_BEGIN_NAMESPACE

typedef QMap<quint32, QString> ServiceMap;

const QDBusArgument &operator>>(const QDBusArgument &argument, ServiceMap &serviceMap);

QT_END_NAMESPACE

QT_DECL_METATYPE_EXTERN(ServiceMap, Q_BLUETOOTH_EXPORT)

#endif // SERVICEMAP_P_H
